# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-09-24 11:29
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ACPHoliday',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date', models.DateField(unique=True, verbose_name='Date')),
            ],
            options={
                'ordering': ('date',),
            },
        ),
        migrations.CreateModel(
            name='HolidayType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=60, verbose_name='Holiday name')),
                ('short_name', models.CharField(max_length=60, unique=True, verbose_name='short name')),
            ],
        ),
        migrations.AddField(
            model_name='acpholiday',
            name='holiday_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='acp_calendar.HolidayType', verbose_name='Holiday type'),
        ),
    ]
